@@ -9,7 +9,7 @@
 #
 package Config::Model::Tester;
 # ABSTRACT: Test framework for Config::Model
-$Config::Model::Tester::VERSION = '2.055';
+$Config::Model::Tester::VERSION = '2.057';
 use warnings;
 use strict;
 use locale;
@@ -190,13 +190,13 @@ sub apply_fix {
     ok( 1, "apply_fixes called" );
 }
 
-sub dump_tree_full_mode {
-    my ($app_to_test, $root, $t, $trace) = @_;
+sub dump_tree {
+    my ($app_to_test, $root, $mode, $no_warnings, $t, $trace) = @_;
 
     print "dumping tree ...\n" if $trace;
     my $dump  = '';
     my $risky = sub {
-        $dump = $root->dump_tree( mode => 'full' );
+        $dump = $root->dump_tree( mode => $mode );
     };
 
     if ( defined $t->{dump_errors} ) {
@@ -211,7 +211,7 @@ sub dump_tree_full_mode {
         }
     }
 
-    if ( ($t->{no_warnings} or (exists $t->{dump_warnings}) and not defined $t->{dump_warnings}) ) {
+    if ( ($no_warnings or (exists $t->{dump_warnings}) and not defined $t->{dump_warnings}) ) {
         local $Config::Model::Value::nowarning = 1;
         &$risky;
         ok( 1, "Ran dump_tree (no warning check)" );
@@ -219,24 +219,12 @@ sub dump_tree_full_mode {
     else {
         warnings_like { &$risky; } $t->{dump_warnings}, "Ran dump_tree";
     }
-    ok( $dump, "Dumped $app_to_test config tree in full mode" );
+    ok( $dump, "Dumped $app_to_test config tree in $mode mode" );
 
     print $dump if $trace;
     return $dump;
 }
 
-# TODO: factorise with function above and create parameters to
-# handle warnings in both cases with warnings_like
-sub dump_tree_custom_mode {
-    my ($label, $root, $t, $trace) = @_;
-
-    local $Config::Model::Value::nowarning = $t->{no_warnings} || 0;
-
-    my $dump = $root->dump_tree();
-    ok( $dump, "Dumped $label config tree in custom mode" );
-    return $dump;
-}
-
 sub check_data {
     my ($label, $root, $c, $nw) = @_;
 
@@ -410,7 +398,7 @@ sub run_model_test {
 
     note("Beginning $app_to_test test ($app_to_test_conf)");
 
-    unless ( my $return = do $app_to_test_conf ) {
+    unless ( my $return = do "./$app_to_test_conf" ) {
         warn "couldn't parse $app_to_test_conf: $@" if $@;
         warn "couldn't do $app_to_test_conf: $!" unless defined $return;
         warn "couldn't run $app_to_test_conf" unless $return;
@@ -444,6 +432,7 @@ sub run_model_test {
 
     my $idx = 0;
     foreach my $t (@tests) {
+        translate_test_data($t);
         my $t_name = $t->{name} || "t$idx";
         if ( defined $do and $t_name !~ /$do/) {
             $idx++;
@@ -474,11 +463,14 @@ sub run_model_test {
 
         load_instructions ($root,$t,$trace) if $t->{load} ;
 
+        dump_tree ('before fix '.$app_to_test , $root, 'full', $t->{no_warnings}, $t->{check_before_fix}, $trace)
+            if $t->{check_before_fix};
+
         apply_fix($inst) if  $t->{apply_fix};
 
-        dump_tree_full_mode ($app_to_test, $root, $t, $trace) ;
+        dump_tree ($app_to_test, $root, 'full', $t->{no_warnings}, $t->{full_dump}, $trace) ;
 
-        my $dump = dump_tree_custom_mode ($app_to_test, $root, $t, $trace) ;
+        my $dump = dump_tree ($app_to_test, $root, 'custom', $t->{no_warnings}, {}, $trace) ;
 
         check_data("first", $root, $t->{check}, $t->{no_warnings}) if $t->{check};
 
@@ -495,7 +487,7 @@ sub run_model_test {
 
         my $i2_root = create_second_instance ($app_to_test, $t_name, $wr_dir, $wr_dir2,$t, $config_dir_override);
 
-        my $p2_dump = dump_tree_custom_mode("second $app_to_test", $i2_root, $t) ;
+        my $p2_dump = dump_tree("second $app_to_test", $i2_root, 'custom', $t->{no_warnings},{}) ;
 
         unified_diff;
         eq_or_diff(
@@ -518,6 +510,11 @@ sub run_model_test {
 
 }
 
+sub translate_test_data {
+    my $t = shift;
+    map {$t->{full_dump}{$_} = delete $t->{$_} if $t->{$_}; } qw/dump_warnings dump_errors/;
+}
+
 sub run_tests {
     my ( $arg, $test_only_app, $do ) = @_;
 
@@ -575,7 +572,7 @@ Config::Model::Tester - Test framework for Config::Model
 
 =head1 VERSION
 
-version 2.055
+version 2.057
 
 =head1 SYNOPSIS
 
@@ -831,6 +828,41 @@ See L<Config::Model::Loader> for the syntax of the string accepted by C<load> pa
 
 =item *
 
+Optionally, run a check before running apply_fix (if any). This step is useful to check
+warning messages:
+
+   check_before_fix => {
+      dump_errors   => [ ... ] # optional, see below
+      dump_warnings => [ ... ] # optional, see below
+   }
+
+Use C<dump_errors> if you expect issues:
+
+  check_before_fix => {
+    dump_errors =>  [
+        # the issues  and a way to fix the issue using Config::Model::Node::load
+        qr/mandatory/ => 'Files:"*" Copyright:0="(c) foobar"',
+        qr/mandatory/ => ' License:FOO text="foo bar" ! Files:"*" License short_name="FOO" '
+    ],
+  }
+
+Likewise, specify any expected warnings (note the list must contain
+only ref to regular expressions):
+
+  check_before_fix => {
+        dump_warnings => [ (qr/deprecated/) x 3 ],
+  }
+
+You can tolerate any dump warning this way:
+
+  check_before_fix => {
+        dump_warnings => undef ,
+  }
+
+Both C<dump_warnings> and C<dump_errors> can be specified in C<check_before_fix> hash.
+
+=item *
+
 Optionally, call L<apply_fixes|Config::Model::Instance/apply_fixes>:
 
     apply_fix => 1,
@@ -838,23 +870,9 @@ Optionally, call L<apply_fixes|Config::Model::Instance/apply_fixes>:
 =item *
 
 Call L<dump_tree|Config::Model::Node/dump_tree ( ... )> to check the validity of the
-data. Use C<dump_errors> if you expect issues:
+data after optional C<apply_fix>. This step is not optional.
 
-    dump_errors =>  [
-        # the issues     the fix that will be applied
-        qr/mandatory/ => 'Files:"*" Copyright:0="(c) foobar"',
-        qr/mandatory/ => ' License:FOO text="foo bar" ! Files:"*" License short_name="FOO" '
-    ],
-
-=item *
-
-Likewise, specify any expected warnings (note the list must contain only C<qr> stuff):
-
-        dump_warnings => [ (qr/deprecated/) x 3 ],
-
-You can tolerate any dump warning this way:
-
-        dump_warnings => undef ,
+As with C<check_before_fix>, both C<dump_errors> or C<dump_warnings> can be used.
 
 =item *
 
